"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("../../config");
const throwError_1 = __importDefault(require("../../utils/throwError"));
const getKeyType_1 = __importDefault(require("../../utils/getKeyType"));
const getKeyType_2 = require("../../utils/getKeyType/getKeyType");
const getKeysFromRoutePattern_1 = __importDefault(require("../../utils/getKeysFromRoutePattern"));
const generateRoutePatternFile = (params) => {
    const { routePattern, routeName, destinationDir, routingType, origin } = params;
    const keys = getKeysFromRoutePattern_1.default(routePattern);
    const patternName = `pattern${routeName}`;
    const originName = `origin${routeName}`;
    const filename = patternName;
    const pathParams = generatePathParamsInterface(keys, routeName);
    const possiblePathParams = generatePossiblePathParams(keys, routeName);
    const urlParts = generateUrlPartsInterface(routeName, pathParams);
    const patternNextJS = routingType === config_1.RoutingType.NextJS ? generateNextJSPattern({ keys, routeName, routePattern }) : null;
    const pathParamsNextJS = routingType === config_1.RoutingType.NextJS ? generateNextJSPathParams(keys, routeName) : null;
    const template = `export const ${patternName} = '${routePattern}'
  export const ${originName} = '${origin}'
  ${patternNextJS ? patternNextJS.template : ""}
  ${pathParams ? pathParams.template : ""}
  ${pathParamsNextJS ? pathParamsNextJS.template : ""}
  ${possiblePathParams ? possiblePathParams.template : ""}
  ${urlParts.template}`;
    const result = [
        {
            template,
            filename,
            extension: ".ts",
            destinationDir,
        },
        {
            originName,
            patternName,
            patternNameNextJS: patternNextJS ? patternNextJS.variableName : undefined,
            pathParamsInterfaceName: pathParams ? pathParams.interfaceName : undefined,
            pathParamsInterfaceNameNextJS: pathParamsNextJS ? pathParamsNextJS.interfaceName : undefined,
            possiblePathParamsVariableName: possiblePathParams ? possiblePathParams.variableName : undefined,
            urlPartsInterfaceName: urlParts.interfaceName,
            filename,
        },
    ];
    return result;
};
const generatePathParamsInterface = (keys, routeName) => {
    if (keys.length === 0) {
        return;
    }
    const pathParamsInterfaceName = `PathParams${routeName}`;
    let template = `export type ${pathParamsInterfaceName} = {`;
    keys.forEach((key) => {
        const { pattern, name, modifier } = key;
        const fieldName = `${name}${modifier === "?" ? modifier : ""}`;
        switch (getKeyType_1.default(key)) {
            case getKeyType_2.KeyType.normal:
                template += `${fieldName}: string;`;
                break;
            case getKeyType_2.KeyType.enum:
                const enumArray = pattern.split("|");
                if (enumArray.length > 0) {
                    template += `${fieldName}:`;
                    enumArray.forEach((enumValue) => (template += `'${enumValue}'|`));
                    template = template.slice(0, -1);
                    template += `;`;
                }
                break;
        }
    });
    template += "}";
    return {
        template,
        interfaceName: pathParamsInterfaceName,
    };
};
const generatePossiblePathParams = (keys, routeName) => {
    if (keys.length === 0) {
        return;
    }
    const variableName = `possilePathParams${routeName}`;
    let template = `export const ${variableName} = [`;
    template = keys.reduce((prevTemplate, { name }) => prevTemplate + `'${name}',`, template);
    template += "]";
    return {
        template,
        variableName,
    };
};
const generateNextJSPathParams = (keys, routeName) => {
    if (keys.length === 0) {
        return null;
    }
    const pathParamsInterfaceName = `PathParamsNextJS${routeName}`;
    let template = `export interface ${pathParamsInterfaceName} {`;
    keys.forEach((key) => {
        const { name, modifier } = key;
        const fieldName = `${name}${modifier === "?" ? modifier : ""}`;
        template += `${fieldName}: string;`;
    });
    template += "}";
    return {
        template,
        interfaceName: pathParamsInterfaceName,
    };
};
const generateUrlPartsInterface = (routeName, pathParams) => {
    const interfaceName = `UrlParts${routeName}`;
    const template = `export interface ${interfaceName} {
    ${pathParams ? `path: ${pathParams.interfaceName};` : ""}
    query?: Record<string, string | undefined>;
    origin?: string;
  }`;
    return { template, interfaceName };
};
const generateNextJSPattern = (params) => {
    const { keys, routeName, routePattern } = params;
    const variableName = `patternNextJS${routeName}`;
    const routeParts = routePattern.split("/");
    const routePartsNextJS = routeParts.map((routePart) => {
        if (routePart.charAt(0) !== ":") {
            return routePart;
        }
        const matchedKey = keys.find((key) => {
            switch (getKeyType_1.default(key)) {
                case getKeyType_2.KeyType.normal:
                    return routePart === `:${key.name}${key.modifier}`;
                case getKeyType_2.KeyType.enum:
                    return routePart === `:${key.name}(${key.pattern})${key.modifier}`;
                default:
                    return false;
            }
        });
        if (!matchedKey) {
            return throwError_1.default([], `Cannot find key for ${routePart} in ${routePattern}`);
        }
        return `[${matchedKey.name}]`;
    });
    const template = `export const ${variableName} = '${routePartsNextJS.join("/")}'`;
    return { template, variableName };
};
exports.default = generateRoutePatternFile;
//# sourceMappingURL=generatePatternFile.js.map